package org.rcisoft.business.device.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.*;
import org.rcisoft.business.device.entity.BusDevice;
import org.rcisoft.business.device.service.BusDeviceService;
import org.rcisoft.business.device.vo.DeviceAssetStatisticVo;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
* Created by  on 2018-4-12 15:33:23.
*/
@RestController
@RequestMapping("busdevice")
public class BusDeviceController extends PaginationController<BusDevice> {

    @Autowired
    private BusDeviceService busDeviceServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@Valid BusDevice busDevice, BindingResult bindingResult) {
        PersistModel data = busDeviceServiceImpl.save(busDevice);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busDevice);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @DeleteMapping("/delete/{id:\\w+}")
    public Result delete(@PathVariable String id) {
        BusDevice busDevice = new BusDevice();
        PersistModel data = busDeviceServiceImpl.remove(busDevice);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{id:\\w+}")
    public Result update(@Valid BusDevice busDevice, BindingResult bindingResult) {
        PersistModel data = busDeviceServiceImpl.merge(busDevice);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busDevice);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @GetMapping("/detail/{id:\\w+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busDeviceServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看  集合")
    @GetMapping(value = "/queryBusDeviceByPagination")
    public GridModel listByPagination(BusDevice busDevice) {
//        busDevice.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        busDeviceServiceImpl.findAllByPagination(getPaginationUtility(), busDevice);
        return getGridModelResponse();
    }

    /**
     * 通过项目id和系统id获取该子系统中的所有设备信息
      * @param sysId    系统id
     * @param proId     项目id
     * @return
     */
    @ApiOperation(value = "通过项目id与子系统id获取子系统下所有的设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysId", value = "sysId", required = true,paramType = "query", dataType = "varchar"),
            @ApiImplicitParam(name = "proId", value = "proId", required = true,paramType = "query", dataType = "varchar")
            })
    @ResponseBody
    @RequestMapping(value = "/getDeviceBySysId",method = RequestMethod.GET)
    public Result getDeviceBySysId(@RequestParam("sysId") String sysId,
                                   @RequestParam("proId") String proId)
    {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busDeviceServiceImpl.getDevicesBySysId(sysId, proId));
    }

    @ApiOperation(value = "通过项目id与子系统id导出子系统的设备清单excel文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysId", value = "sysId", required = true,paramType = "path", dataType = "varchar"),
            @ApiImplicitParam(name = "proId", value = "proId", required = true,paramType = "path", dataType = "varvhar")
    })
    @ResponseBody
    @RequestMapping(value = "/exportExcel/{proId}/{sysId}",method = RequestMethod.GET)
    public void exportExcel(HttpServletResponse response,@PathVariable("sysId") String sysId,
                            @PathVariable("proId") String proId) throws UnsupportedEncodingException {
        System.out.println("参数："+sysId+"  "+proId);
        List<DeviceAssetStatisticVo> deviceList = new ArrayList<>();
        //先获取数据库数据
        deviceList = busDeviceServiceImpl.getDevicesBySysId(sysId, proId);
        //通过项目id与系统id获取对应的名称组拼成文件名
        String filename = busDeviceServiceImpl.getProNameByProId(proId) + busDeviceServiceImpl.getSysNameBySysId(sysId)+"设备统计报表.xls";

        //创建excel生成对象
        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet("设备统计表");

        //表头内容
        String[] headers = { "设备名称", "设备类型", "设备型号", "设备厂家","设备参数","安装时间"};
        HSSFRow row = sheet.createRow(0);
        //在excel表中添加表头
        for(int i=0;i<headers.length;i++){
            HSSFCell cell = row.createCell(i);
            HSSFRichTextString text = new HSSFRichTextString(headers[i]);
            cell.setCellValue(text);
        }
        //日期类型格式化
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        //在表中存放查询到的数据放入对应的列
        int rowNum = 1;
        for (DeviceAssetStatisticVo temp :
                deviceList) {
            HSSFRow rowTemp = sheet.createRow(rowNum);
            rowTemp.createCell(0).setCellValue(temp.getDevNm());
            rowTemp.createCell(1).setCellValue(temp.getDevTpNm());
            rowTemp.createCell(2).setCellValue(temp.getModel());
            rowTemp.createCell(3).setCellValue(temp.getFacNm());
            rowTemp.createCell(4).setCellValue(temp.getParam());
            rowTemp.createCell(5).setCellValue(sdf.format(temp.getInstallDate()));
            rowNum++;
        }
        //设置响应头信息
        response.setHeader("Content-disposition", "attachment;filename="+URLEncoder.encode(filename, "utf-8"));
        try {
            OutputStream outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



}
