package org.rcisoft.business.overview.dao;

import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.overview.entity.EnergyCountM;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created by JiChao on 2018/4/11.
 * 概览
 */
@Repository
public interface EnergyCountMRepository extends BaseMapper<EnergyCountM> {

    /**
     * 用量和费用统计，当日或当月
     * @param energyCountM proId：项目id；day：天；mon：月；year：年
     * @return
     */
    @Select("<script>select sum(WATER) as water,sum(WATER_MONEY) as waterMoney," +
            "sum(ELEC) as elec,sum(ELEC_MONEY) as elecMoney," +
            "sum(GAS) as gas,sum(GAS_MONEY) as gasMoney" +
            " from ENERGY_COUNT_M e " +
            "where e.PRO_ID=#{proId} and `year`=#{year}" +
            "<if test=\"day != null\"> and `day`=#{day} group by e.`day`</if>" +
            "<if test=\"mon != null\"> and mon=#{mon} group by e.mon</if></script>")
    @ResultMap("BaseResultMap")
    EnergyCountM countAmountAndPrice(EnergyCountM energyCountM);

    /**
     * 运行费用统计
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.HOUR," +
            "<if test=\"type == 1\">sum(e.WATER_MONEY)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC_MONEY)</if>" +
            "<if test=\"type == 3\">sum(e.GAS_MONEY)</if>" +
            " as MONEY from ENERGY_COUNT_M e" +
            " where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon} and e.`DAY`=#{day} group by e.HOUR</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> countDailyPrice(EnergyCountMVo energyCountMVo);

    /**
     * 耗能拆分统计
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.DEV_ID," +
            "<if test=\"type == 1\">sum(e.WATER)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC)</if>" +
            "<if test=\"type == 3\">sum(e.GAS)</if>" +
            " as ENERGY from ENERGY_COUNT_M e" +
            " where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon} and e.`DAY`=#{day} group by e.DEV_ID</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> countEnergySplit(EnergyCountMVo energyCountMVo);

    /**
     * 运行费用排名
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select a.* from (" +
            "select e.`DAY`," +
            "<if test=\"type == 1\">sum(e.WATER_MONEY)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC_MONEY)</if>" +
            "<if test=\"type == 3\">sum(e.GAS_MONEY)</if>" +
            " as MONEY from ENERGY_COUNT_M e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon}" +
            "<if test=\"hour != null\"> and e.HOUR &lt;= #{hour}</if>" +
            " group by e.`DAY`) a order by a.MONEY desc</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> getPriceRank(EnergyCountMVo energyCountMVo);

    /**
     * 用能比较
     * @param energyCountMVo
     * @return
     */
    @Select("<script>select e.`DAY`," +
            "<if test=\"type == 1\">sum(e.WATER)</if>" +
            "<if test=\"type == 2\">sum(e.ELEC)</if>" +
            "<if test=\"type == 3\">sum(e.GAS)</if>" +
            " as ENERGY from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon} group by e.`DAY`</script>")
    @ResultMap("resultMapVo")
    List<EnergyCountMVo> getEnergyCompare(EnergyCountMVo energyCountMVo);

    /**
     * 按月统计水电气能耗
     * @param energyCountM proId：项目主键，year：年份
     * @return
     */
    @Select("<script>select e.MON,sum(e.WATER) as WATER,sum(e.ELEC) as ELEC,sum(e.GAS) as GAS from energy_count_m e" +
            " where e.PRO_ID=#{proId} and e.`YEAR`=#{year} group by e.MON</script>")
    @ResultMap("BaseResultMap")
    List<EnergyCountM> getEnergyMonth(EnergyCountM energyCountM);

    /**
     * 得到每天水电气的能耗
     * @param energyCountM
     * @return
     */
    @Select("<script>select SUM(e.WATER) WATER,SUM(e.ELEC) ELEC,SUM(e.GAS) GAS from energy_count_m e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon} and e.`DAY`=#{day}</script>")
    @ResultMap("BaseResultMap")
    EnergyCountM countEnergyDay(EnergyCountM energyCountM);

}
