package org.rcisoft.business.overview.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.overview.dao.*;
import org.rcisoft.business.overview.entity.*;
import org.rcisoft.business.overview.service.OverViewService;
import org.rcisoft.business.overview.vo.BusProjectAreaVo;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.business.overview.vo.EnergyPriceVo;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.*;

/**
 * Created by JiChao on 2018/4/11.
 */
@Service
@Slf4j
public class OverViewServiceImpl implements OverViewService {

    @Autowired
    private EnergyPriceRepository overViewRepository;
    @Autowired
    private EnergyCountMRepository energyCountMRepository;
    @Autowired
    private BusProjectAreaRepository busProjectAreaRepository;
    @Autowired
    private BusProjectAreaStatRepository busProjectAreaStatRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private EnergyEmissionRealRepository energyEmissionRealRepository;
    @Autowired
    private EnergyEmissionTotalRepository energyEmissionTotalRepository;
    @Autowired
    private EnergyEmissionPlanRepository energyEmissionPlanRepository;

    private Calendar getBeforeDay(Calendar cl, int time){
        int day = cl.get(Calendar.DATE);
        cl.set(Calendar.DATE, day + time);
        return cl;
    }

    //返回昨日时间
    private Map<String, String> getTime(int time){
        Calendar now = Calendar.getInstance();
        if(time != 0) now = this.getBeforeDay(now, time);
        Map<String, String> result = new HashMap<String, String>();
        result.put("year", String.valueOf(now.get(Calendar.YEAR)));
        result.put("month", String.valueOf(now.get(Calendar.MONTH) + 1));
        result.put("day", String.valueOf(now.get(Calendar.DAY_OF_MONTH)));
        result.put("hour", String.valueOf(now.get(Calendar.HOUR_OF_DAY)));
        return result;
    }

    //返回同比或环比的年、月
    private Map<String, String> getTime(String year, String month, String day, int yearVar, int monthVar, int dayVar){
        Calendar calendar = Calendar.getInstance();
        if(year != null){
            calendar.set(Calendar.YEAR, Integer.parseInt(year) + yearVar);
        }
        if(month != null){
            calendar.set(Calendar.MONTH, Integer.parseInt(month) - 1 + monthVar);
        }
        if(day != null){
            calendar.set(Calendar.DATE, Integer.parseInt(day) + dayVar);
        }
        Map<String, String> result = new HashMap<String, String>();
        result.put("year", String.valueOf(calendar.get(Calendar.YEAR)));
        result.put("month", String.valueOf(calendar.get(Calendar.MONTH) + 1));
        result.put("day", String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
        return result;
    }

    public static void main(String[] args){
        OverViewServiceImpl o = new OverViewServiceImpl();
        Map<String, String> time = o.getTime("2016", "3", null, 0, -1, 0);
        System.out.println(time.get("year") + "-" + time.get("month") + "-" + time.get("day"));
    }

    private String getType(String type) {
        switch (type) {
            case "1": //水
                type = "water";
                break;
            case "2": //电
                type = "elec";
                break;
            case "3": //气
                type = "gas";
                break;
            default:
                break;
        }
        return type;
    }

    private List<Float> getFloatList(List<EnergyCountMVo> list){
        List<Float> resultList = new ArrayList<Float>();
        if(list != null && list.size() > 0){
            int size = list.size();
//            Integer start = Integer.parseInt(list.get(0).getDay());
//            Integer end = Integer.parseInt(list.get(size - 1).getDay());
//            for (int i = 1; i < start; i++) {
//                resultList.add(0f);
//            }
//            for (int i = 0; i < size; i++) {
//                resultList.add(list.get(i).getEnergy());
//            }
//            for (int i = end; i < 31; i++) {
//                resultList.add(0f);
//            }
            int j = 1;
            for (int i = 0; i < size; i++) {
                EnergyCountMVo e = list.get(i);
                int day = Integer.parseInt(e.getDay());
                while (day > j++){
                    resultList.add(0f);
                }
                resultList.add(e.getEnergy());
                if(i == size - 1){
                    while (day++ < 31){
                        resultList.add(0f);
                    }
                }
            }
        }else{
            for (int i = 0; i < 31; i++) {
                resultList.add(0f);
            }
        }
        return resultList;
    }

    //月传12，日传31
    private List<Float> getEmissionResult(List<EnergyEmissionReal> list, int num) {
        List<Float> resultList = new ArrayList<Float>();
        if(list != null && list.size() > 0){
            int size = list.size();
//            Integer start = num == 12 ? list.get(0).getMon() : list.get(0).getDay();
//            Integer end = num == 12 ? list.get(size - 1).getMon() : list.get(size - 1).getDay();
//            for (int i = 1; i < start; i++) {
//                resultList.add(0f);
//            }
//            for (int i = 0; i < size; i++) {
//                resultList.add(list.get(i).getEmission());
//            }
//            for (int i = end; i < num; i++) {
//                resultList.add(0f);
//            }
            int j = 1;
            for (int i = 0; i < size; i++) {
                EnergyEmissionReal e = list.get(i);
                int temp = num == 12 ? e.getMon() : e.getDay();
                while (temp > j++){
                    resultList.add(0f);
                }
                resultList.add(e.getEmission());
                if(i == size - 1){
                    while (temp++ < num){
                        resultList.add(0f);
                    }
                }
            }
        }else{
            for (int i = 0; i < num; i++) {
                resultList.add(0f);
            }
        }
        return resultList;
    }

    @Override
    public EnergyPriceVo getPrice(String proId) {
        Map<String, Object> map = new HashMap<String, Object>();
        int i = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
        EnergyPrice e = new EnergyPrice();
        e.setProId(proId);
        e.setPHour(String.valueOf(i));
        return overViewRepository.getPrice(e);
    }

    @Override
    public Map<String, Object> countAmountAndPrice(String proId) {
        Map<String, String> cal = this.getTime(0);
        EnergyCountM m = new EnergyCountM();
        m.setProId(proId);
        m.setYear(cal.get("year"));
        //统计日
        m.setDay(cal.get("day"));
        EnergyCountM dayCount = energyCountMRepository.countAmountAndPrice(m);
        //统计月
        m.setDay(null);
        m.setMon(cal.get("month"));
        EnergyCountM monthCount = energyCountMRepository.countAmountAndPrice(m);
        //合并返回值
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("day", dayCount);
        result.put("mon", monthCount);
        return result;
    }

    @Override
    public List<EnergyCountMVo> countDailyPrice(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(cal.get("day"));
        return energyCountMRepository.countDailyPrice(e);
    }

    @Override
    public List<EnergyCountMVo> countEnergySplit(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(cal.get("day"));
        return energyCountMRepository.countEnergySplit(e);
    }

    @Override
    public Integer getPriceRank(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        String day = cal.get("day");
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(day);
        if(date != 0) e.setHour(cal.get("hour"));
        List<EnergyCountMVo> list = energyCountMRepository.getPriceRank(e);
        for (int i = 0; i < list.size(); i++) {
            EnergyCountMVo energyCountMVo = list.get(i);
            String eDay = energyCountMVo.getDay();
            if(StringUtils.equals(day, eDay))
                return i + 1;
        }
        return 1;
    }

    @Override
    public Map<String, Object> getStandard(String proId) {
        BusProjectArea b = new BusProjectArea();
        b.setProId(proId);
        List<BusProjectArea> list = busProjectAreaRepository.queryBusProjectAreas(b);
        //返回值
        Map<String, Object> result = new HashMap<String, Object>();
        Iterator<BusProjectArea> it = list.iterator();
        while (it.hasNext()) {
            BusProjectArea busProjectArea = it.next();
            String type = this.getType(busProjectArea.getType());
            if(StringUtils.isNotEmpty(type)) result.put(type, busProjectArea);
        }
        return result;
    }

    @Override
    public Map<String, Object> getUnitArea(String proId) {
        String thisY = getTime(0).get("year");
        int i = Integer.parseInt(thisY);
        String lastY = String.valueOf(i - 1);
        String beforeY = String.valueOf(i - 2);
        BusProjectAreaStat b = new BusProjectAreaStat();
        b.setProId(proId);
        //查询当年
        b.setYear(thisY);
        BusProjectAreaStat thisBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //查询去年
        b.setYear(lastY);
        BusProjectAreaStat lastBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //查询前年
        b.setYear(beforeY);
        BusProjectAreaStat beforeBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //返回结果
        Map<String, Object> result = new HashMap<String, Object>();
        if(beforeBus == null) {
            beforeBus = new BusProjectAreaStat(beforeY, 0f, 0f, 0f);
        }
        if(lastBus == null) {
            lastBus = new BusProjectAreaStat(beforeY, 0f, 0f, 0f);
        }
        if(thisBus == null) {
            thisBus = lastBus;
        }
        result.put("thisY", thisBus);
        result.put("lastY", lastBus);
        result.put("beforeY", beforeBus);
        return result;
    }

    @Override
    public Map<String, Object> getEnergyCompare(String proId, String year, String month, Integer type, Integer cType) {
        //返回值
        Map<String, Object> result = new HashMap<String, Object>();
        //添加当前年月
        result.put("year", year);
        result.put("month", month);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(year);
        e.setMon(month);
        //查找现在的数据
        List<EnergyCountMVo> nowCompare = energyCountMRepository.getEnergyCompare(e);
        //现在数据最后返回的对象
        List<Float> nowReturn = this.getFloatList(nowCompare);
        //查找需要比较的数据
        Map<String, String> time = null;
        //同比，环比，确定时间
        if(cType == 1){//同比
            time = this.getTime(year, month, null, -1, 0, 0);
        }else{//环比
            time = this.getTime(year, month, null, 0, -1, 0);
        }
        //新的时间
        String c_year = time.get("year");
        String c_month = time.get("month");
        e.setYear(c_year);
        e.setMon(c_month);
        //添加当前年月
        result.put("cYear", c_year);
        result.put("cMonth", c_month);
        List<EnergyCountMVo> lastCompare = energyCountMRepository.getEnergyCompare(e);
        List<Float> lastReturn = this.getFloatList(lastCompare);
        //差值
        List<Float> diffReturn = new ArrayList<Float>();
        for (int i = 0; i < nowReturn.size(); i++) {
            diffReturn.add(nowReturn.get(i) - lastReturn.get(i));
        }
        //数据值
        result.put("nData", nowReturn);
        result.put("cData", lastReturn);
        result.put("dData", diffReturn);
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel statEnergyMonthly() {
        System.out.println("---------------------start--------------------");
        int result = 0;
        //需要更新或保存的list
        List<BusProjectAreaStat> resultList = new ArrayList<BusProjectAreaStat>();
        //得到需要统计的年份
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        if(calendar.get(Calendar.MONTH) == 1) year -= 1;
        String yearS = String.valueOf(year);
        //1.得到所有的项目
        List<BusProject> busProjectList = busProjectRepository.queryBusProjects();
        //2.循环所有的项目，查询对应项目的统计信息
        Iterator<BusProject> busProjectIterator = busProjectList.iterator();
        while (busProjectIterator.hasNext()) {
            //得到项目主键
            String proId = busProjectIterator.next().getProId();
            //查询该项目的水电气面积
            BusProjectAreaVo busProjectAreaVo = busProjectAreaRepository.classifyProjectArea(proId);
            //3.查询对应项目的能源统计（按月份统计）
            EnergyCountM e = new EnergyCountM();
            e.setProId(proId);
            e.setYear(yearS);
            List<EnergyCountM> energyList = energyCountMRepository.getEnergyMonth(e);
            //4.算出水电气的统计
            if(energyList != null){
                int size = energyList.size();
                if(size > 0){
                    //得到全年总和
                    Float waterSum = 0f, elecSum = 0f, gasSum = 0f;
                    Iterator<EnergyCountM> it = energyList.iterator();
                    while (it.hasNext()) {
                        EnergyCountM energyCountM = it.next();
                        waterSum += energyCountM.getWater();
                        elecSum += energyCountM.getElec();
                        gasSum += energyCountM.getGas();
                    }
                    //算出年度平均值
                    waterSum = waterSum * 12 / size / busProjectAreaVo.getWaterArea();
                    elecSum = elecSum * 12 / size / busProjectAreaVo.getElecArea();
                    gasSum = gasSum * 12 / size / busProjectAreaVo.getGasArea();
                    //需要新增或更新的对象
                    resultList.add(new BusProjectAreaStat(proId, yearS, waterSum, elecSum, gasSum));
                }
            }
        }
        //增加或更新
        Iterator<BusProjectAreaStat> resultIt = resultList.iterator();
        while (resultIt.hasNext()) {
            BusProjectAreaStat busProjectAreaStat = resultIt.next();
            Example example = new Example(BusProjectAreaStat.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("year", yearS);
            int i = busProjectAreaStatRepository.updateByExampleSelective(busProjectAreaStat, example);
            if (i < 1) {
                i = busProjectAreaStatRepository.insert(busProjectAreaStat);
            }
            result += i;
        }
        System.out.println("---------------------end--------------------");
        return new PersistModel(result);
    }

    @Override
    public Map<String, Object> getEmission(String proId) {
        Map<String, Object> result = new HashMap<String, Object>();
        Calendar calendar = this.getBeforeDay(Calendar.getInstance(), -1);
        //昨天时间
        Integer year = calendar.get(Calendar.YEAR), month = calendar.get(Calendar.MONTH) + 1, day = calendar.get(Calendar.DAY_OF_MONTH);
        //分8次查询
        //查询当日，当月，当年的数据
        //当年
        EnergyEmissionReal e_now = new EnergyEmissionReal(proId, year);
        EnergyEmissionReal year_now = energyEmissionRealRepository.statEnergyEmissionReal(e_now);
        if(year_now != null) result.put("year_now", year_now.getEmission());
        else result.put("year_now", 0);
        //当月
        e_now.setMon(month);
        EnergyEmissionReal month_now = energyEmissionRealRepository.statEnergyEmissionReal(e_now);
        if(month_now != null) result.put("month_now", month_now.getEmission());
        else result.put("month_now", 0);
        //当日
        e_now.setDay(day);
        EnergyEmissionReal day_now = energyEmissionRealRepository.queryEnergyEmissionReal(e_now);
        if(day_now != null) result.put("day_now", day_now.getEmission());
        else result.put("day_now", 0);
        //同比年
        EnergyEmissionReal e_same = new EnergyEmissionReal(proId, year - 1);
        EnergyEmissionReal year_same = energyEmissionRealRepository.statEnergyEmissionReal(e_same);
        if(year_same != null) result.put("year_same", (Float) result.get("year_now") - year_same.getEmission());
        else result.put("year_same", (Float) result.get("year_now") - 0);
        //同比月
        e_same.setMon(month);
        EnergyEmissionReal month_same = energyEmissionRealRepository.statEnergyEmissionReal(e_same);
        if(month_same != null) result.put("month_same", (Float) result.get("month_now") - month_same.getEmission());
        else result.put("month_same", (Float) result.get("month_now") - 0);
        //同比日
        e_same.setDay(day);
        EnergyEmissionReal day_same = energyEmissionRealRepository.queryEnergyEmissionReal(e_same);
        if(day_same != null) result.put("day_same", (Float) result.get("day_now") - day_same.getEmission());
        else result.put("day_same", (Float) result.get("day_now") - 0);
        //环比月
        Map<String, String> ring_time = this.getTime(null, month.toString(), null, 0, -1, 0);
        EnergyEmissionReal e_ring = new EnergyEmissionReal(proId, Integer.parseInt(ring_time.get("year")), Integer.parseInt(ring_time.get("month")), Integer.parseInt(ring_time.get("day")));
        EnergyEmissionReal month_ring = energyEmissionRealRepository.statEnergyEmissionReal(e_ring);
        if(month_ring != null) result.put("month_ring", (Float) result.get("month_now") - month_ring.getEmission());
        else result.put("month_ring", (Float) result.get("month_now") - 0);
        //环比日
        ring_time = this.getTime(null, null, day.toString(), 0, 0, -1);
        e_ring = new EnergyEmissionReal(proId, Integer.parseInt(ring_time.get("year")), Integer.parseInt(ring_time.get("month")), Integer.parseInt(ring_time.get("day")));
        EnergyEmissionReal day_ring = energyEmissionRealRepository.queryEnergyEmissionReal(e_ring);
        if(day_ring != null) result.put("day_ring", (Float) result.get("day_now") - day_ring.getEmission());
        else result.put("day_ring", (Float) result.get("day_now") - 0);
        return result;
    }

    @Override
    public EnergyEmissionTotal getEmissionStat(String proId) {
        int year = Calendar.getInstance().get(Calendar.YEAR);
        EnergyEmissionTotal e = new EnergyEmissionTotal(proId, year);
        return energyEmissionTotalRepository.queryEnergyEmissionTotals(e);
    }

    @Override
    public List<Float> statEmissionMonth(String proId, int year) {
        EnergyEmissionReal e = new EnergyEmissionReal(proId, year);
        List<EnergyEmissionReal> list = energyEmissionRealRepository.statEmissionMonth(e);
        return this.getEmissionResult(list, 12);
    }

    @Override
    public List<Float> statEmissionDay(String proId, int year, int month) {
        return this.getEmissionResult(energyEmissionRealRepository.statEmissionDay(new EnergyEmissionReal(proId, year, month)), 31);
    }

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel statEmission() {
        int result = 0;
        Calendar calendar = this.getBeforeDay(Calendar.getInstance(), -1);
        //昨天时间
        Integer year = calendar.get(Calendar.YEAR), month = calendar.get(Calendar.MONTH) + 1, day = calendar.get(Calendar.DAY_OF_MONTH);
        //1.得到所有的项目
        List<BusProject> busProjectList = busProjectRepository.queryBusProjects();
        //2.循环所有的项目，查询对应项目的统计信息
        Iterator<BusProject> busProjectIterator = busProjectList.iterator();
        while (busProjectIterator.hasNext()) {
            //得到项目主键
            String proId = busProjectIterator.next().getProId();
            EnergyCountM energyCountM = energyCountMRepository.countEnergyDay(new EnergyCountM(proId, day.toString(), month.toString(), year.toString()));
            if(energyCountM != null){
                //得到碳排放量
                Float water = energyCountM.getWater(), elec = energyCountM.getElec(), gas = energyCountM.getGas();
                float total = (float) (Math.floor(((water == null ? 0f : water * 0.91f) + (elec == null ? 0f : elec * 0.785f) + (gas == null ? 0f : gas * 0.19f)) * 100) / 100);
                //保存每天的实际碳排放量
                EnergyEmissionReal e = new EnergyEmissionReal(null, proId, year, month, day, total);
                int i = energyEmissionRealRepository.insert(e);
                if (i > 0) {
                    EnergyEmissionTotal t = energyEmissionTotalRepository.selectOne(new EnergyEmissionTotal(null, proId, null, year));
                    if(t != null){
                        EnergyEmissionPlan p = energyEmissionPlanRepository.selectOne(new EnergyEmissionPlan(null, proId, month, day, null));
                        Float pEmission = p == null ? 0f : p.getEmission();
                        t.setEmission(pEmission - total + t.getEmission());
                        Example example = new Example(BusProjectAreaStat.class);
                        Example.Criteria criteria = example.createCriteria();
                        criteria.andEqualTo("id", t.getId());
                        result += energyEmissionTotalRepository.updateByExample(t, example);
                    }else{
                        t = new EnergyEmissionTotal(null, proId, total, year);
                        result += energyEmissionTotalRepository.insert(t);
                    }
                }
            }
        }
        return new PersistModel(result);
    }

}
